%{

/*
 *  NICE - utility
 *
 *  Programmed: Arne Vajhj (july 1994)
 *
 */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#ifdef __DECC
#include <unixio.h>
#endif

#include "nice.h"
#define EXTERN extern
#include "glob.h"

extern nokeywrd;
extern char *keywrd[];
extern int token[];

int lnr = 1;    /* linenumber  */
int save = -1;  /* saved token */

/* work variables */
int i;
int c;
char *p;

/*  small tricks  */
#define malloc nice_malloc
#define realloc nice_realloc
#define free nice_free
#define yylex yylex_generated

%}

D                   [0-9]
H                   [0-9A-Fa-f]
L                   [A-Za-z$_]
A                   [A-Za-z0-9$_]

%%

{D}+                { yylval.ival = atoi(yytext);
                      return INTCONST; } 
{D}+{H}*            { yylval.sval = (char *) nice_malloc(yyleng+1);
                      strcpy(yylval.sval,yytext);
                      return HEXCONST; }
{D}*\.{D}+          { yylval.xval = atof(yytext);
                      return REALCONST; } 
\'(([^\'])|(\'\'))*\'   { yylval.sval = (char *) nice_malloc(yyleng+1);
                      strcpy(yylval.sval,yytext);
                      return STRCONST; }
{L}+{A}*            { yylval.name = (char *) nice_malloc(yyleng+1);
                      cvtupp(yytext,yylval.name);
                      for (i=0;i<nokeywrd;i++) {
                         if (strcmp(keywrd[i],yylval.name)==0) {
                            nice_free(yylval.name);
                            if (token[i]==END && misssemi) {
                               save = END;
                               c = ';';
                               return c;
                            } else {
                               return token[i];
                            }
                         }
                      }
                      idcvt(yylval.name);
                      return ID; }
"(*"                { yylval.sval = (char *) nice_malloc(10000);
                      p = yylval.sval;
                      do {
                         while ((c=input())!='*'){
                            if (c=='\n') lnr++;
                            *p = c;
                            p++;
                         }
                         c=input();
                         if (c!=')') {
                            *p = '*';
                            p++;
                            unput(c);
                         }
                      } while (c!=')');
                      *p = '\0';
                      return COMMENT; }
"{"                 { yylval.sval = (char *) nice_malloc(10000);
                      p = yylval.sval;
                      while ((c=input())!='}') {
                         if (c=='\n') lnr++;
                         *p = c;
                         p++;
                      }
                      *p = '\0';
                      return COMMENT; }
":="                { return ASG; }
"<>"                { return NE; }
">="                { return GE; }
"<="                { return LE; }
"**"                { return POW; }
".."                { return DOT2; }
"#"                 { return NUMB; }
"("                 { return '('; }
")"                 { return ')'; }
"["                 { return '['; }
"]"                 { return ']'; }
"<"                 { return '<'; }
">"                 { return '>'; }
"="                 { return '='; }
":"                 { return ':'; }
"+"                 { return '+'; }
"-"                 { return '-'; }
"*"                 { return '*'; }
"/"                 { return '/'; }
","                 { return ','; }
"."                 { return '.'; }
"^"                 { return '^'; }
";"                 { return ';'; }
"%"                 { return '%'; }
"@"                 { return '@'; }
"\n"                { lnr++;
                      return -1; }
[ \t\f]             { return -1; }
.                   { fprintf(stderr,"ERROR: Lexical error in line %d\n",lnr);
                      return 0; }

%%

#undef yylex

int yylex()
{
   int c;
   char *p;
   if (save>=0) {
                c = save;
                save = -1;
                return c;
                }
   if (fin) return 0;
   do {
      c = yylex_generated();
      if (c==';') {
         misssemi = FALSE;
      } else {
         if (c>=0) misssemi = TRUE;
      };
      if (strcmp(yytext,"\n")==0) {
          buf[0] = '\0';
      } else {
         if (c!=COMMENT) {
            strcat(buf,yytext);
         } else {
            p = yylval.sval + strlen(yylval.sval) - 1;
            while ((p>=yylval.sval) && (*p!='\n')) p--;
            strcat(buf,p);
         }
      }
   } while(c<0);
   return c;
}

int yyerror (char *s)
{
   char *p;
   fprintf(stderr,"ERROR: Parser error\n");
   for(p=buf;*p!='\0';p++) fprintf(stderr,"%c",*p);
   fprintf(stderr,"\n");
   for(p=(char *)&buf[1];*p!='\0';p++) fprintf(stderr," ");
   fprintf (stderr,"^%s in line %d\n",s,lnr);
   return 0;
}

void transerr(char *item)
{
   fprintf(stderr,"\"%s\" can not be translated in line\n",item,lnr);
   return;
}

int yywrap()
{
	return 1;
}
